(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(*  This example is based on an example designed for OCamlP3l.         *)
(*                                                                     *)
(***********************************************************************)

(* $Id: dd_config.ml,v 1.3 2017/01/04 11:58:30 fclement Exp $ *)

module Nat = Unatural;;

open Rdec2;;

(* Graphical output. *)

type graphical_output =
  | No
  | Postscript
  | Window
;;

let graphical_output_of_string = function
  | "no" | "none" -> No
  | "ps" | "postscript" -> Postscript
  | "window" -> Window
  | s -> failwith (Printf.sprintf "Dd_config: Unknown graphical output %s" s)

and string_of_graphical_output = function
  | No -> "none"
  | Postscript -> "postscript"
  | Window -> "window"
;;

(* Configuration. *)

type t =
  {
   rdec_config_filename : string;
   result_basename : string;
   result_dir : string;
   worker_dir : string;
   worker_exe : string;
   graphical_output : graphical_output;
   outer_itermax : length;
   inner_itermax : length;
   display_step : index;
   a_while : length;
  }
and config = t
;;

let configuration_of s1 s2 s3 s4 s5 fname i1 i2 i3 i4 =
  let ib =
    try Scanf.Scanning.from_file fname with
    | Sys_error s -> prerr_endline s; exit 6 in
  let go = Scanf.bscanf ib "%s" graphical_output_of_string in
  Scanf.Scanning.close_in ib;
  {
   rdec_config_filename = s1;
   result_basename = s2;
   result_dir = s3;
   worker_dir = s4;
   worker_exe = s5;
   graphical_output = go;
   outer_itermax = Nat.of_int i1;
   inner_itermax = Nat.of_int i2;
   display_step = Nat.of_int i3;
   a_while = Nat.of_int i4;
  }
;;

let parse_configuration_file fname =
  let ib =
    try Scanf.Scanning.from_file fname with
    | Sys_error s -> prerr_endline s; exit 6 in
  Scanf.bscanf ib
    "rdec_config_filename = %S; \
     result_basename = %S; \
     result_dir = %S; \
     worker_dir = %S; \
     worker_exe = %S; \
     graphical_output = %S; \
     outer_itermax = %d; \
     inner_itermax = %d; \
     display_step = %d; \
     a_while = %d; "
    configuration_of
;;

let print_configuration_file ob cfg =
  Printf.bprintf ob "\
     rdec_config_filename = %S;\n\
     result_basename = %S;\n\
     result_dir = %S;\n\
     worker_dir = %S;\n\
     worker_exe = %S;\n\
     graphical_output = %S;\n\
     outer_itermax = %d;\n\
     inner_itermax = %d;\n\
     display_step = %d;\n\
     a_while = %d;\n"
    cfg.rdec_config_filename
    cfg.result_basename
    cfg.result_dir
    cfg.worker_dir
    cfg.worker_exe
    (string_of_graphical_output cfg.graphical_output)
    (Nat.to_int cfg.outer_itermax)
    (Nat.to_int cfg.inner_itermax)
    (Nat.to_int cfg.display_step)
    (Nat.to_int cfg.a_while)
;;

let string_of_configuration_file cfg =
  let ob = Buffer.create 1024 in
  print_configuration_file ob cfg;
  Buffer.contents ob
;;
