(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(*  This example is based on an example designed for OCamlP3l.         *)
(*                                                                     *)
(***********************************************************************)

(* $Id: dd_config.mli,v 1.4 2017/01/04 11:58:30 fclement Exp $ *)

(** {3 Parsing configuration file for the coupling program}

 A configuration file must contain (in that order):
 - the name of the file defining the regular decomposition;
 - the basename for numerical files;
 - the graphical output (window, postscript, none);
 - the maximum number of iterations for the domain decomposition algorithm;
 - the maximum number of iterations for the local subproblems;
 - the refreshment step for the ascii and graphical displays;
 - the number of seconds to sleep before terminating. *)

open Rdec2;;

(** {6 Graphical output} *)

type graphical_output =
  | No
  | Postscript
  | Window
(** The type of graphical outputs. *)
;;

val graphical_output_of_string : string -> graphical_output;;
val string_of_graphical_output : graphical_output -> string;;

(** {6 Configuration} *)

type t =
  {
   rdec_config_filename : string;
   result_basename : string;
   result_dir : string;
   worker_dir : string;
   worker_exe : string;
   graphical_output : graphical_output;
   outer_itermax : length;
   inner_itermax : length;
   display_step : index;
   a_while : length;
  }
and config = t
(** The type of configurations. *)
;;

val parse_configuration_file : string -> t
(** Parsing a configuration file. *)
;;

val print_configuration_file : Buffer.t -> t -> unit
(** Printing a configuration file. *)
;;

val string_of_configuration_file : t -> string
(** string of a configuration file. *)
;;
