(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(*  This example is based on an example designed for OCamlP3l.         *)
(*                                                                     *)
(***********************************************************************)

(* $Id: make_dd1.ml,v 1.1 2017/01/05 16:23:14 fclement Exp $ *)

module Nat = Unatural;;
module A = Uarray;;
module U = Uunix;;

let _0_ = Nat._0_
and _1_ = Nat._1_
and _2_ = Nat._2_
;;

open Rdec2;;
open Subdomain;;
open Dd_config;;
open Gnuplot;;

let set_data rdec =
  let intf0 = make_zero_interface rdec in
  A.map init intf0
;;

let init_poisson subd jtermax (_, oc) =
  let g = computational_rectangular_grid_of_subdomain subd in
  print_computational_rectangular_grid oc g;
  Printf.fprintf oc "%d\n" (Nat.to_int jtermax);
  flush oc
;;

let subd_send_msg cfg (_, oc) i n bd =
  let res = Filename.concat cfg.result_dir cfg.result_basename in
  let go =
    match cfg.graphical_output with
    | No -> "."
    | Postscript
    | Window ->
        if Nat.rem i cfg.display_step <> _0_ then "." else
        Printf.sprintf "%s%d" res (Nat.to_int n) in
  Printf.fprintf oc "%s\n" go;
  print_boundary oc bd;
  flush oc;
;;

let subd_receive_result (ib, _) sz =
  let err = Scanf.bscanf ib " %f %f\n" (fun e1 e2 -> (e1, e2)) in
  let bd = read_boundary ib sz in
  { bd; err; }
;;

let plot_send_msg cfg rdec io_ref iter =
  let go = cfg.graphical_output in
  let bname = Filename.concat cfg.result_dir cfg.result_basename in
  match go with
  | No -> ()
  | Postscript
  | Window ->
    let (_, oc) = U.spawn "gnuplot" io_ref in
    if Nat.rem iter cfg.display_step = _0_ then begin
      let nsbd = number_of_subdomains_of_regular_decomposition rdec in
      let gpl = make_plot_command go bname iter nsbd in
      Printf.fprintf oc "%s\n" gpl;
      flush oc;
    end;
;;

let conv_print_msg cfg v iter =
  let go = cfg.graphical_output in
  let (rel_stag, rel_err) = get_max_errs v in
  Printf.eprintf
    "iter = %2d -> rel_stag = %5.0e, rel_err = %6.2f%%\n"
    (Nat.to_int iter) rel_stag (100. *. rel_err);
  flush stderr;
  match go with
  | No
  | Postscript -> ()
  | Window ->
    (* To give time to Gnuplot to be available. *)
    if Nat.rem iter (Nat.div cfg.outer_itermax _2_) = _0_ then
      Unix.sleep 1;
;;

let print_result _cfg =
  Printf.eprintf "\nComputation from configuration file %S completed.\n\n"
;;
