(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(*  This example is based on an example designed for OCamlP3l.         *)
(*                                                                     *)
(***********************************************************************)

(* $Id: subdomain.ml,v 1.5 2017/01/05 16:00:22 fclement Exp $ *)

(* Operations on subdomains. *)

module A = Uarray;;

open Rdec2;;

type t =
  {
   bd : boundary;
   err : float * float;
  }
and subdomain = t
;;

(* Extraction. *)

let get_bds = A.map (fun subd -> subd.bd)
and get_errs = A.map (fun subd -> subd.err)
;;

let max_err (rs1, re1) { err = (rs2, re2); _; } =
  (max rs1 rs2, max re1 re2)
;;

let get_max_errs = A.fold_left max_err (0., 0.);;

(* Insertion. *)

let set_bds bds v =
  (* Arrays bds and v are assumed to be of the same length.
     Therefore, the next A.unsafe_get will be safe. *)
  A.fill_i (fun i subd -> { subd with bd = A.unsafe_get bds i; }) v
;;

let init bd = { bd; err = (max_float, max_float); };;
