(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: options.ml,v 1.27 2017/01/04 11:58:31 fclement Exp $ *)

module Nat = Unatural;;
module A = Uarray;;
module Ar = Uaref;;
module Ok = Uoption_key;;
module Co = Ucommand_option;;

let _0_ = Nat._0_;;

type running_mode =
  | Std
  | Tri
;;

type t =
  {
   vector_file : string;
   matrix_file : string;
   running_mode : running_mode;
  }
and options = t
;;

let { Ar.get = get_vector_file; set = set_vector_file; } =
  Ar.make_aref_single "vector_file" "data/demo_vec"

and { Ar.get = get_matrix_file; set = set_matrix_file; } =
  Ar.make_aref_single "matrix_file" "data/demo_mat"

and { Ar.get = get_running_mode; set = set_running_mode; } =
  Ar.make_aref_single "running_mode" "std"
;;

let mode_of_string = function
  | "std" -> Std
  | "tri" -> Tri
  | _ -> assert false
;;

let add_options () =
  let access_ok f s =
    try Unix.access s [ Unix.R_OK; ]; f s with
    | Unix.Unix_error _ ->
      Printf.eprintf "Cannot access %S for reading.\n" s;
      exit 2 in

  Co.application_add_all_rco_auto
    [
     {
      Co.key  = Ok.of_string "-vector-file";
      spec = Arg.String (access_ok set_vector_file);
      doc =
        Printf.sprintf
          "<file>  Set the file containing the input vector.\
          \n\tDefault is %S."
          (get_vector_file ());
     };

     {
      Co.key  = Ok.of_string "-matrix-file";
      spec = Arg.String (access_ok set_matrix_file);
      doc =
        Printf.sprintf
          "<file> Sets the file containing the input matrix.\
          \n\tDefault is %S."
          (get_matrix_file ());
     };

     {
      Co.key  = Ok.of_string "-mode";
      spec = Arg.Symbol ([ "std"; "tri"; ], set_running_mode);
      doc =
        Printf.sprintf
          "<mode>  Set the running mode of the demo.\
          \n\tDefault is %s."
          (get_running_mode ());
     };
    ];
;;
add_options ();;

(* Sys.argv is never empty.
   Therefore, the next A.unsafe_get will be safe. *)
let cmd_name = A.unsafe_get Sys.argv _0_;;
Co.set_usage_msg
  (Printf.sprintf "\n%s -- Matrice-vector product" cmd_name)
;;

let get_options () =
  {
   vector_file = get_vector_file ();
   matrix_file = get_matrix_file ();
   running_mode = mode_of_string (get_running_mode ());
  }
;;
