(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: parser.mli,v 1.10 2017/01/04 11:58:31 fclement Exp $ *)

(* Monadic parsers *)
(* Copyright (c) Quentin Carbonneaux 2011 *)

type ('a, 'b) t = { parse : 'a -> ('b * 'a) option }
and ('a, 'b) parser = ('a, 'b) t
;;

val parse : ('a, 'b) t -> 'a -> 'b option;;
val full_parse : ('a, 'b) t -> 'a -> ('b * 'a) option;;

val return : 'b -> ('a, 'b) t;;
val zero : ('a, 'b) t;;
val bind : ('a, 'b) t -> ('b -> ('a, 'c) t) -> ('a, 'c) t;;
val plus : ('a, 'b) t -> ('a, 'b) t -> ('a, 'b) t;;
val ( >>= ) : ('a, 'b) t -> ('b -> ('a, 'c) t) -> ('a, 'c) t;;
val ( >> ) : ('a, 'b) t -> ('a, 'c) t -> ('a, 'c) t;;
val ( +++ ) : ('a, 'b) t -> ('a, 'b) t -> ('a, 'b) t;;

val liftM : ('a -> 'b) -> ('c, 'a) t -> ('c, 'b) t;;

val any : (('a, 'b) t) list -> ('a, 'b) t;;
val many : ('a, 'b) t -> ('a, 'b list) t;;
val many1 : ('a, 'b) t -> ('a, 'b list) t;;
val many_till : ('a, 'b) t -> ('a, 'c) t -> ('a, 'c list) t;;
val times : int -> ('a, 'b) t -> ('a, 'b list) t;;
val sep_by1 : ('a, unit) t -> ('a, 'b) t -> ('a, 'b list) t;;
val less_than : int -> ('a, 'b) t -> ('a, 'b list) t;;
val maybe : ('a, 'b) t -> ('a, 'b option) t;;
