(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: options.ml,v 1.15 2017/01/04 11:58:31 fclement Exp $ *)

module Nat = Unatural;;
module Perv = Upervasives;;
module A = Uarray;;
module S = Ustring;;
module Ar = Uaref;;
module Ok = Uoption_key;;
module Co = Ucommand_option;;

let _0_ = Nat._0_;;

let qual = Perv.qualify "Options";;
let invalid_arg = Perv.invalid_arg_qual qual "";;

let external_program_command_option_name = "external program command"
and init_external_program_command = []
;;

let { Ar.get_list = get_option_external_program_command;
      add_item = add_option_external_program_command_item; _; } =
  Ar.make_aref_list init_external_program_command
;;

let { Ok.long = external_program_command_lok;
      short = external_program_command_sok_st; } =
  Ok.make_option_keys
    external_program_command_option_name
    (Ok.Short (Ok.of_string "--"))
;;

let option_external_program_command =
  {
   Co.key = external_program_command_lok;
   spec = Arg.Rest add_option_external_program_command_item;
   doc =
     Printf.sprintf
       "<cmd> [<option>...] [<argument>...]\n\
        \tSet the path to the external program command to <cmd>.\n\
        \tAlso set optional options and arguments.\n\
        \tThis must be the last arguments of the present command."
  }
;;

let add_options () =
  Co.application_add_all_rco_with_sk
    [
     (external_program_command_sok_st, option_external_program_command);
    ]
;;

add_options ();;

(* Sys.argv is never empty.
   Therefore, the next A.unsafe_get will be safe. *)
let cmd_name = A.unsafe_get Sys.argv _0_;;
Co.set_usage_msg
  (Format.sprintf
     "@[<v>\
      @ \
      %s -- Sklml example with the use of Pio communications@ \
      @ \
      Usage:@ \
      @ \
      %s [<option>...] -- <cmd> [<cmd_option>...] [<cmd_argument>...]@ \
      @ \
      Options are:\
      @]@."
     cmd_name
     cmd_name)
;;

let get_external_program_command () =
  match get_option_external_program_command () with
  | [] -> invalid_arg "empty external program command path"
  | _ :: _ as sl -> S.concat " " sl
;;
