(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: expor.ml,v 1.24 2017/01/04 11:58:27 fclement Exp $ *)

module Nat = Unatural;;

let _0_ = Nat._0_
and _32_ = Nat.unsafe_of_int 32
;;

module A = Darray;;

open Common_types;;

type ('x, 'a, 'b) t = atom_id
and ('x, 'a, 'b) expor = ('x, 'a, 'b) t
;;

let closures = A.create _32_;;

let register (f : 'x -> 'a Sklstream.t -> 'b Sklstream.t) =
  let x = Obj.repr f in
  (A.append closures x : ('x, 'a, 'b) t)
;;

let retreive (i : ('x, 'a, 'b) t) =
  let x = A.get closures i in
  (Obj.magic x : 'x -> 'a Sklstream.t -> 'b Sklstream.t)
;;

let unsafe_retreive : atom_id -> 'x -> 'a Sklstream.t -> 'b Sklstream.t =
  retreive
;;

let get_exportable (params, x) =
  let p = Obj.repr params in
  (p, x)
;;

type ('a, 'b) either =
  | Right of 'a
  | Left of 'b
;;

let is_registered f =
  let g a b =
    match a with
    | Right i -> Right i
    | Left i -> if b == Obj.repr f then Right i else Left (Nat.succ i) in
  match A.fold_left g (Left _0_) closures with
  | Right i -> Some i
  | Left _ -> None
;;
