(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: main.ml,v 1.35 2017/01/04 11:58:29 fclement Exp $ *)

(* Sklml parallel launcher daemon.

 This independant program can run in two modes: master or slave.
 In master mode, it will be contacted by a master Sklml program to create a
 computational network. Its different slaves will let him spawn the nodes of
 the computational network on several processors.
 In slave mode, it will be connected to a master daemon and spawn processes on
 its processor.

 With the options [-mode master -slave-number N] where [N] is an integer,
 it will run in master mode, waiting for [N] slaves.
 With the options [-mode slave -master-address ADDR] where [ADDR] is a valid
 IPv4 address, it will run in slave mode, and will connect to the master node
 listening on [ADDR] and act for it. *)

open Tools_common_types;;
open Node_types;;
open Node_types_print;;

let { tagged_debug = debug; _; } = Options.services;;

let debug fname = debug (Printf.sprintf "Main.%s" fname);;

let parse () =
  try Options.parse () with
  | Arg.Bad msg ->
    Spawnlib.fail (Printf.sprintf "Bad argument (use -help option): %s" msg)
  | Arg.Help msg -> Printf.eprintf "%s" msg; exit 0
  | Failure msg -> Spawnlib.fail msg
;;

let main () =
  let debug = debug "main" in
  let node = parse () in
  debug "command line parsed";
  debug (Options.sprint_ident_value "node" print_node node);
  match node with
  | Master_node mn -> Master.start mn
  | Slave_node sn -> Slave.start sn
;;

main ();;
