(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: log.ml,v 1.18 2017/01/04 11:58:29 fclement Exp $ *)

module Nat = Unatural;;
module A = Uarray;;

let _0_ = Nat._0_
and _1_ = Nat._1_
;;

type verbosity_level = Nat.t;;

type log_level =
  | Err
  | Log of verbosity_level
  | Say
  | Warn
;;

type ('a, 'b) ks_format4 = ('a, unit, string, 'b) format4;;

(* Sys.argv is never empty.
   Therefore, the next A.unsafe_get will be safe. *)
let me = Filename.basename (A.unsafe_get Sys.argv _0_);;

let prerr_not_nat var =
  prerr_endline (Printf.sprintf "%s: %s must be a natural." me var)
;;

let get_nat_env var default =
  try Nat.of_string (Sys.getenv var) with
  | Failure _ -> prerr_not_nat var; default
  | Not_found -> default
;;

let verbosity () = get_nat_env "SKLML_VERBOSITY" _1_;;

let ksay_intern prefix suffix log_level fmt =
  let real_logger oc out =
    let ksuff s = Printf.fprintf oc "%s%s%!" s suffix; out in
    match prefix with
    | None -> Printf.ksprintf ksuff fmt
    | Some p -> Printf.ksprintf ksuff ("%s: " ^^ fmt) p
  and dummy_logger _ k = Printf.ksprintf (fun _ -> k) fmt in
  let conditional_logger i =
    if verbosity () <= i then dummy_logger else
    real_logger in
  match log_level with
  | Err -> real_logger stderr
  | Log i -> conditional_logger i stderr
  | Say -> real_logger stdout
  | Warn -> conditional_logger _0_ stderr
;;

let say level fmt = ksay_intern (Some me) "\n" level fmt ();;

let progress level fmt =
  let final fmt = ksay_intern None "\n" level fmt () in
  ksay_intern (Some me) "" level fmt final
;;
