(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: parfun.ml,v 1.36 2017/01/04 11:58:29 fclement Exp $ *)

module A = Uarray;;

type t =
  { parfun : 'a 'b. ('a, 'b) Skl.t -> 'a Sklstream.t -> 'b Sklstream.t; }
and parfun = t
;;

let rec parfun :
    'a 'b. ('a, 'b) Skl.t -> 'a Sklstream.t -> 'b Sklstream.t = function

  | Sk.Skl.Atom atom ->
    let f_atom params llexp _ =
      let f_gen = Expor.retreive llexp in
      let f = f_gen params in
      fun x -> f x in
    Sk.with_atom atom { Sk.f_atom; }

  | Sk.Skl.Id id_skl ->
    let f_id f = Sklstream.map f in
    Sk.with_id id_skl { Sk.f_id; }

  | Sk.Skl.Loop loop ->
    let f_loop coerce pskl skl =
      let p = parfun pskl in
      let f = parfun skl in
      fun x -> Sklstream.map coerce (Sklstream.loop p f x) in
      Sk.with_loop loop { Sk.f_loop; }

  | Sk.Skl.Par par ->
    let f_par skl1 skl2 =
      let f1 = parfun skl1 in
      let f2 = parfun skl2 in
      fun x -> Sklstream.in_par (f1, f2) x in
    Sk.with_par par { Sk.f_par; }

  | Sk.Skl.Pipe pipe ->
    let f_pipe skl1 skl2 =
      let f1 = parfun skl1 in
      let f2 = parfun skl2 in
      fun x -> f2 (f1 x) in
    Sk.with_pipe pipe { Sk.f_pipe; }

  | Sk.Skl.Prod prod ->
    let f_prod coerce_in coerce_out skl1 skl2 =
      let f1 = parfun skl1 in
      let f2 = parfun skl2 in
      fun x ->
        Sklstream.map coerce_out
          (Sklstream.in_prod (f1, f2) (Sklstream.map coerce_in x)) in
    Sk.with_prod prod { Sk.f_prod; }

  | Sk.Skl.Rails rails ->
    let f_rails ska =
      let fv = A.map parfun ska in
      fun x -> Sklstream.on_rails fv x in
    Sk.with_rails rails { Sk.f_rails; }

  | Sk.Skl.Sum sum ->
    let f_sum coerce_in skl1 skl2 =
      let f1 = parfun skl1 in
      let f2 = parfun skl2 in
      fun x -> Sklstream.in_sum (f1, f2) (Sklstream.map coerce_in x) in
    Sk.with_sum sum { Sk.f_sum; }
;;

let pardo f = f { parfun; };;
(* Entry point *)
