(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: error.ml,v 1.16 2017/01/04 11:58:30 fclement Exp $ *)

(* Batch add fail. *)

module Nat = Unatural;;
module L = Ulist;;
module U = Uunix;;

let nat_uunix_kill_and_wait pid = U.kill_and_wait (Nat.to_int pid);;

let batch_add_fail_dead_slave () =
  Batch.add_fail_task Batch.empty
    "One slave died before starting computation, aborting"
;;

let batch_add_fail_exited () =
  Batch.add_fail_task Batch.empty
    "Sklml parallel launcher daemon exited prematurely, aborting"
;;

let batch_add_fail_no_port line =
  Batch.add_fail_task Batch.empty
    (Printf.sprintf
       "Sklml parallel launcher daemon sent invalid data: %S, aborting"
       line)
;;

let batch_add_fail_timeout () =
  Batch.add_fail_task Batch.empty
    "Waiting for Sklml parallel launcher daemon response \
    for too long, aborting"
;;

(* Batch fail action. *)

let batch_fail_client = Batch.Fail_error "Client code failed";;

let batch_fail_launch_client =
  Batch.Fail_error "Failed to launch the client Sklml code"
;;

let batch_fail_launch_slave name =
  Batch.Fail_error (Printf.sprintf "Failed to launch slave %S" name)
;;

let batch_fail_network = Batch.Fail_error "Parallel network failed";;

let batch_fail_start_master name =
  Batch.Fail_error
    (Printf.sprintf "Failed to start master node %S" name)
;;

let { Tools_common_types.tagged_warning = warning; tagged_error = error; _; } =
  Options.services
;;

(* Error. *)

let error_empty_command () =
  error "invalid configuration: empty slave program command"
;;

let error_expand_shell_command msg =
  error (Printf.sprintf "Error, invalid format: %S\n\n" msg)
;;

let error_kill_jobs pidl =
  L.iter nat_uunix_kill_and_wait pidl;
  let err_msg =
    Printf.sprintf "There were errors, %d jobs killed.\n"
      (Nat.to_int (L.length pidl)) in
  error err_msg
;;

let error_parse_conf msg =
  error (Printf.sprintf "Error, parsing error in config near:\n%s\n\n" msg)
;;

(* Warning. *)

let warn_duplicated_entry k =
  warning (Printf.sprintf "duplicated entry %S" k)
;;

let warn_expected_int k =
  warning (Printf.sprintf "entry %S was expected of type int" k)
;;

let warn_expected_string k =
  warning (Printf.sprintf "entry %S was expected of type string" k)
;;

let warn_meaningless k =
  warning (Printf.sprintf "entry %S is meaningless in the master section" k)
;;
