(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: error.mli,v 1.10 2017/01/04 11:58:30 fclement Exp $ *)

open Tools_common_types;;

(** {6 Batch add fail} *)

val batch_add_fail_dead_slave : unit -> Batch.t;;
val batch_add_fail_exited : unit -> Batch.t;;
val batch_add_fail_no_port : string -> Batch.t;;
val batch_add_fail_timeout : unit -> Batch.t;;

(** {6 Batch fail action} *)

val batch_fail_client : Batch.fail_action;;
val batch_fail_launch_client : Batch.fail_action;;
val batch_fail_launch_slave : string -> Batch.fail_action;;
val batch_fail_network : Batch.fail_action;;
val batch_fail_start_master : string -> Batch.fail_action;;

(** {6 Error} *)

val error_empty_command : unit -> 'a;;
val error_expand_shell_command : string -> 'a;;
val error_kill_jobs : pid list -> 'a;;
val error_parse_conf : string -> 'a;;

(** {6 Warning} *)

val warn_duplicated_entry : string -> unit;;
val warn_expected_int : string -> unit;;
val warn_expected_string : string -> unit;;
val warn_meaningless : string -> unit;;
