(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: sklml_path.ml,v 1.6 2017/01/04 11:58:30 fclement Exp $ *)

open Upph_id_card;;

open Sklml_install_data;;
open Tools_common_types;;

let get_compiler_and_lib_suffix = function
  | Byt -> (ocaml_compilers.byt, ".cma")
  | Bin -> (ocaml_compilers.bin, ".cmxa")
;;

let get_generator = function
  | Dep -> ocaml_compilers.dep
  | Doc -> ocaml_compilers.doc
;;

let system_libs = [ "bigarray"; "unix"; ];;

let pph_libs = [ "ulib"; ];;

let pph_include_dir dir lib =
  mk_path dir [ "private"; "lib"; "ocaml"; lib; ]
;;

let sklml_compilation_include_dirs dir =
  [
   mk_path dir [ "tools_common"; ];
   mk_path dir [ "common"; ];
   mk_path dir [ "extra"; ];
  ]
;;

let sklml_libs = function
  | Seq -> [ "tools_common"; "common"; "sklml"; "sklml_extra"; ]
  | Par -> [ "tools_common"; "common"; "parlib"; "sklml"; "sklml_extra"; ]
;;

let sklml_link_include_dirs dir = function
  | Seq ->
      [
       mk_path dir [ "tools_common"; ];
       mk_path dir [ "common"; ];
       mk_path dir [ "sequential"; "lib"; ];
       mk_path dir [ "extra"; ];
      ]
  | Par ->
      [
       mk_path dir [ "tools_common"; ];
       mk_path dir [ "common"; ];
       mk_path dir [ "parallel"; "parlib"; ];
       mk_path dir [ "parallel"; "lib"; ];
       mk_path dir [ "extra"; ];
      ]
;;

let project_distribution_name =
  Sklml_version.pph_id_card.project_distribution_name
;;

let get_sklml_lib_root_dir ird =
  mk_path ird [ "lib"; "ocaml"; project_distribution_name; ]

and get_sklml_pph_tekno_dir ird =
  mk_path ird [ "share"; project_distribution_name; "config"; ]
;;
